/* -*- mia-c++  -*-
 *
 * This file is part of MIA - a toolbox for medical image analysis 
 * Copyright (c) Leipzig, Madrid 1999-2013 Gert Wollny
 *
 * MIA is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MIA; if not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef reg3d_deformer_hh
#define reg3d_deformer_hh

#include <memory>
#include <mia/3d/image.hh>
#include <mia/3d/filter.hh>
#include <mia/3d/interpolator.hh>


NS_MIA_BEGIN 

/**
   @ingroup registration 
   @brief A filter to transform  an image
   @remark obsolete, should be done by using C3DTransformation
*/

struct FDeformer3D: public TFilter<P3DImage> {
	FDeformer3D(const C3DFVectorfield& vf, const C3DInterpolatorFactory& ipfac): 
		m_vf(vf), 
		m_ipfac(ipfac)
		{
		}
	template <typename T> 
	P3DImage operator () (const T3DImage<T>& image) const {
		T3DImage<T> *timage = new T3DImage<T>(image.get_size(), image); 
		P3DImage result(timage); 
		std::auto_ptr<T3DInterpolator<T> > interp(m_ipfac.create(image.data())); 
		
#ifdef __OPENMP                
#pragma omp parallel for
#endif                
		for (size_t z = 0; z < image.get_size().z; ++z) {
			typename T3DImage<T>::iterator r = timage->begin_at(0,0,z); 
			C3DFVectorfield::const_iterator v = m_vf.begin_at(0,0,z); 
			for (size_t y = 0; y < image.get_size().y; ++y)
				for (size_t x = 0; x < image.get_size().x; ++x, ++r, ++v)
					*r = (*interp)(C3DFVector(x - v->x, y - v->y, z - v->z));
		}
		return result; 
	}

	template <typename T> 
	P3DImage operator () (const T3DImage<T>& image, T3DImage<T>& result) const {
		std::auto_ptr<T3DInterpolator<T> > interp(m_ipfac.create(image.data())); 
#ifdef __OPENMP                
#pragma omp parallel for
#endif                
		for (size_t z = 0; z < image.get_size().z; ++z) {
			typename T3DImage<T>::iterator r = result.begin_at(0,0,z); 
			C3DFVectorfield::const_iterator v = m_vf.begin_at(0,0,z); 
			for (size_t y = 0; y < image.get_size().y; ++y)
				for (size_t x = 0; x < image.get_size().x; ++x, ++r, ++v)
					*r = (*interp)(C3DFVector(x - v->x, y - v->y, z - v->z));
		}
		return P3DImage(); 
	}
	
private: 
	C3DFVectorfield m_vf; 
	C3DInterpolatorFactory m_ipfac; 
}; 


NS_MIA_END

#endif
